"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollectionMetadata = void 0;
const common_1 = require("@nestjs/common");
const constants_1 = require("./constants");
class CollectionMetadata {
    entity;
    static registry = new Map();
    logger = new common_1.Logger(CollectionMetadata.name);
    constructor(entity) {
        this.entity = entity;
    }
    async synchronize(collection) {
        const entityMetadata = Reflect.getMetadata(constants_1.ENTITY_METADATA, this.entity.prototype) || {
            columns: []
        };
        const uniqueIndexKeys = [];
        const internalLockIndexKeys = ['_lockPinned'];
        for (const column of entityMetadata.columns) {
            if (column.options.unique) {
                uniqueIndexKeys.push(column.propertyKey);
                continue;
            }
            if (column.options.lock) {
                internalLockIndexKeys.push(column.propertyKey);
                continue;
            }
        }
        if (!await collection.indexExists('_internalLock')) {
            const internalLockSpecification = internalLockIndexKeys.reduce((spec, key) => ({ ...spec, [key]: 1 }), {});
            await collection.createIndex(internalLockSpecification, {
                name: '_internalLock',
                unique: true
            })
                .catch(error => this.logger.debug(error.message, error.stack));
        }
        if (uniqueIndexKeys.length) {
            const uniqueIndexSpecification = uniqueIndexKeys.reduce((spec, key) => ({ ...spec, [key]: 1 }), {});
            await collection.createIndex(uniqueIndexSpecification, {
                unique: true
            })
                .catch(error => this.logger.debug(error.message, error.stack));
        }
    }
    static collect(entity) {
        if (this.registry.has(entity)) {
            return this.registry.get(entity);
        }
        const metadata = new CollectionMetadata(entity);
        return this.registry.set(entity, metadata).get(entity);
    }
}
exports.CollectionMetadata = CollectionMetadata;
//# sourceMappingURL=metadata.js.map